package com.book.server.controller;

import com.book.server.common.entity.PageResult;
import com.book.server.common.entity.Result;
import com.book.server.dao.entity.Book;
import com.book.server.model.VO.BlockRes;
import com.book.server.model.VO.BookRes;
import com.book.server.model.VO.QueryVO;
import com.book.server.service.BookService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 书籍
 */
@RestController
@RequestMapping("/api/book")
@Slf4j
public class BookController extends BaseController {
    @Autowired
    private BookService bookService;


    /**
     * 书库查询
     *
     * @param queryVO
     * @return
     */
    @PostMapping("/query")
    public Result<PageResult<BookRes>> query(@RequestBody QueryVO queryVO) {
        return Result.success(bookService.query(queryVO));
    }

    /**
     * 书城分页推荐
     *
     * @param queryVO
     * @return
     */
    @PostMapping("/recommend")
    public Result<PageResult<Book>> recommendByRead(@RequestBody QueryVO queryVO) {
        return Result.success(bookService.recommendByRead(queryVO));
    }

    /**
     * 书城block
     *
     * @param pageId
     * @return
     */
    @GetMapping("/getBlockByPageId")
    @Cacheable(value = "block", key = "#pageId")
    public Result<List<BlockRes>> getBlockByPageId(Integer pageId) {
        List<BlockRes> list = bookService.getBlockByPageId(pageId);
        return Result.byObject(list);
    }

    /**
     * 更新block缓存
     *
     * @return
     */
    @GetMapping("/resetBlockCache")
    @CacheEvict(value = "block", allEntries = true)
    public Result resetBlockCache() {
        return Result.byObject("OK");
    }

}
