package com.book.server.service.impl;

import com.book.server.common.entity.PageResult;
import com.book.server.common.entity.Result;
import com.book.server.dao.entity.Book;
import com.book.server.dao.entity.BookCategory;
import com.book.server.dao.entity.ManageBlock;
import com.book.server.dao.entity.ManageBlockResource;
import com.book.server.dao.entity.example.BookCategoryExample;
import com.book.server.dao.entity.example.BookExample;
import com.book.server.dao.entity.example.ManageBlockExample;
import com.book.server.dao.entity.example.ManageBlockResourceExample;
import com.book.server.dao.mapper.BookCategoryMapper;
import com.book.server.dao.mapper.BookMapper;
import com.book.server.dao.mapper.ManageBlockMapper;
import com.book.server.dao.mapper.ManageBlockResourceMapper;
import com.book.server.model.VO.BlockRes;
import com.book.server.model.VO.BookRes;
import com.book.server.model.VO.ManageBlockResourceRes;
import com.book.server.model.VO.QueryVO;
import com.book.server.service.BookService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class BookServiceImpl implements BookService {
    @Autowired
    private BookMapper bookMapper;

    @Autowired
    private ManageBlockMapper manageBlockMapper;

    @Autowired
    private ManageBlockResourceMapper manageBlockResourceMapper;
    @Autowired
    private BookCategoryMapper bookCategoryMapper;

    public List<BookCategory> getAllCategory() {
        List<BookCategory> bookCategories = bookCategoryMapper.selectByExample(
                BookCategoryExample.newAndCreateCriteria().example()
        );
        return bookCategories;
    }


    @Override
    public PageResult<BookRes> query(QueryVO queryVO) {
        BookExample example = BookExample.newAndCreateCriteria().example();
        example.or().andNameLike("%" + queryVO.getQuery() + "%");
        example.or().andAuthorLike("%" + queryVO.getQuery() + "%");
        long count = bookMapper.countByExample(example);
        if (count <= 0) {
            return new PageResult<>(queryVO.getPage(), queryVO.getSize(), 0L, null);
        }
        example.page(queryVO.getPage(), queryVO.getSize());
        List<Book> books = bookMapper.selectByExample(example);
        //查询关联表
        List<BookCategory> allCategory = getAllCategory();
        HashMap<Integer, String> map = new HashMap<>();
        for (BookCategory bookCategory : allCategory) {
            map.put(bookCategory.getId(), bookCategory.getName());
        }
        //转存
        ArrayList<BookRes> bookResList = new ArrayList<>();
        for (Book book : books) {
            BookRes bookRes = new BookRes();
            BeanUtils.copyProperties(book,bookRes);
            bookRes.setCategoryName(map.get(book.getBookCategoryId()));
            bookResList.add(bookRes);
        }

        return new PageResult<>(queryVO.getPage(), queryVO.getSize(), count, bookResList);
    }

    @Override
    public PageResult<Book> recommendByRead(QueryVO queryVO) {
        BookExample example = BookExample.newAndCreateCriteria()
                .andSexEqualTo(queryVO.getPageId())
                .example();
        long count = bookMapper.countByExample(example);
        example.orderBy(Book.Column.readNum.desc());
        example.page(queryVO.getPage(), queryVO.getSize());
        List<Book> books = bookMapper.selectByExample(example);
        return new PageResult<>(queryVO.getPage(), queryVO.getSize(), count, books);
    }

    @Override
    public List<BlockRes> getBlockByPageId(Integer pageId) {
        ArrayList<BlockRes> list = new ArrayList<>();
        // 获取板块
        ManageBlockExample manageBlockExample = ManageBlockExample.newAndCreateCriteria()
                .andPageIdEqualTo(pageId)
                .example();
        manageBlockExample.orderBy(ManageBlock.Column.weigh.desc());
        List<ManageBlock> manageBlocks = manageBlockMapper.selectByExample(manageBlockExample);

        // 获取板块内部数据
        for (ManageBlock manageBlock : manageBlocks) {
            ManageBlockResourceExample manageBlockResourceExample = ManageBlockResourceExample.newAndCreateCriteria()
                    .andBlockIdEqualTo(manageBlock.getId())
                    .example();
            manageBlockResourceExample.orderBy(ManageBlockResource.Column.weigh.desc());
            //获取内部书籍资源
            List<ManageBlockResource> manageBlockResources = manageBlockResourceMapper.selectByExample(manageBlockResourceExample);
            ArrayList<ManageBlockResourceRes> manageBlockResourceResList = new ArrayList<>();
            if (manageBlockResources.size() > 0) {
                List<Long> collect = manageBlockResources.stream()
                        .map(ManageBlockResource::getBookId).collect(Collectors.toList());
                List<Book> books = bookMapper.selectByExample(
                        BookExample.newAndCreateCriteria()
                                .andIdIn(collect)
                                .example()
                );
                HashMap<Long, Book> bookHashMap = new HashMap<>();
                books.forEach(x->{
                    bookHashMap.put(x.getId(), x);
                });
                manageBlockResources.forEach(x->{
                    ManageBlockResourceRes manageBlockResourceRes = new ManageBlockResourceRes();
                    BeanUtils.copyProperties(x,manageBlockResourceRes);
                    manageBlockResourceResList.add(manageBlockResourceRes);

                    Book book = bookHashMap.get(x.getBookId());
                    if (book != null) {
                        manageBlockResourceRes.setBookName(book.getName());
                        manageBlockResourceRes.setBookDescription(book.getDescription());
                    }

                });
            }
            BlockRes blockRes = new BlockRes();
            blockRes.setName(manageBlock.getName());
            blockRes.setSecondName(manageBlock.getSecondName());
            blockRes.setType(manageBlock.getType());
            blockRes.setResources(manageBlockResourceResList);

            list.add(blockRes);
        }
        return list;
    }


}
